@@ -1,25 +1,21 @@
-#
-#  xmltv.py - Python interface to XMLTV format, based on XMLTV.pm
-#
-#  Copyright (C) 2001 James Oakley
-#
-#  This library is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU Lesser General Public
-#  License as published by the Free Software Foundation; either
-#  version 2.1 of the License, or (at your option) any later version.
-#
-#  This program is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#  Lesser General Public License for more details.
-#
-#  You should have received a copy of the GNU Lesser General Public
-#  License along with this program; if not, write to the Free Software
-#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+"""
+xmltv.py - Python interface to XMLTV format, based on XMLTV.pm
+
+Copyright (C) 2001 James Oakley <jfunk@funktronics.ca>
+
+This library is free software: you can redistribute it and/or modify it under
+the terms of the GNU Lesser General Public License as published by the Free
+Software Foundation; either version 3 of the License, or (at your option) any
+later version.
+
+This program is distributed in the hope that it will be useful, but WITHOUT ANY
+WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
+PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public License along
+with this software; if not, see <http://www.gnu.org/licenses/>.
+"""
 
-#
-#  If you have any trouble: jfunk@funktronics.ca
-#
 import re
 import sys
 import types
@@ -33,7 +29,7 @@ else:
         from elementtree.ElementTree import ElementTree, Element, SubElement, tostring
 
 # The Python-XMLTV version
-VERSION = "1.2"
+VERSION = "1.4"
 
 # The date format used in XMLTV (the %Z will go away in 0.6)
 date_format = '%Y%m%d%H%M%S %Z'
@@ -155,8 +151,10 @@ def elem_to_programme(elem):
     crednode = elem.find('credits')
     if crednode is not None:
         creddict = {}
+        # TODO: actor can have a 'role' attribute
         for credtype in ('director', 'actor', 'writer', 'adapter', 'producer',
-                         'presenter', 'commentator', 'guest'):
+                         'presenter', 'commentator', 'guest', 'composer',
+                         'editor'):
             append_text(creddict, credtype, crednode, with_lang=False)
         d['credits'] = creddict
 
@@ -230,12 +228,22 @@ def elem_to_programme(elem):
         append_icons(ratd, ratnode)
         d['rating'].append(ratd)
 
-    srnode = elem.find('star-rating')
-    if srnode is not None:
+    for srnode in elem.findall('star-rating'):
+        if not d.has_key('star-rating'):
+            d['star-rating'] = []
         srd = {}
+        set_attrs(srd, srnode, ('system',))
         set_text(srd, 'value', srnode, with_lang=False)
         append_icons(srd, srnode)
-        d['star-rating'] = srd
+        d['star-rating'].append(srd)
+
+    for revnode in elem.findall('review'):
+        if not d.has_key('review'):
+            d['review'] = []
+        rd = {}
+        set_attrs(rd, revnode, ('type', 'source', 'reviewer',))
+        set_text(rd, 'value', revnode, with_lang=False)
+        d['review'].append(rd)
 
     return d
 
@@ -524,11 +532,24 @@ class Writer:
 
         # Star rating
         if programme.has_key('star-rating'):
-            sr = SubElement(p, 'star-rating')
-            v = SubElement(sr, 'value')
-            self.settext(v, programme['star-rating']['value'], with_lang=False)
-            if programme['star-rating'].has_key('icon'):
-                self.seticons(sr, programme['star-rating']['icon'])
+            for star_rating in programme['star-rating']:
+                sr = SubElement(p, 'star-rating')
+                if star_rating.has_key('system'):
+                    self.setattr(sr, 'system', star_rating['system'])
+                v = SubElement(sr, 'value')
+                self.settext(v, star_rating['value'], with_lang=False)
+                if star_rating.has_key('icon'):
+                    self.seticons(sr, rating['icon'])
+
+        # Review
+        if programme.has_key('review'):
+            for review in programme['review']:
+                r = SubElement(p, 'review')
+                for attr in ('type', 'source', 'reviewer'):
+                    if review.has_key(attr):
+                        self.setattr(r, attr, review[attr])
+                v = SubElement(r, 'value')
+                self.settext(v, review['value'], with_lang=False)
 
     def addChannel(self, channel):
         """
@@ -605,6 +626,7 @@ if __name__ == '__main__':
     <credits>
       <actor>Jerry Seinfeld</actor>
       <producer>Larry David</producer>
+      <composer>Jonathan Wolff</composer>
     </credits>
     <date>1995</date>
     <length units="minutes">22</length>
@@ -631,6 +653,9 @@ if __name__ == '__main__':
       <value>4/5</value>
       <icon src="http://some.star/icon.png" width="32" height="32"/>
     </star-rating>
+    <review type="url">
+      <value>http://some.review/</value>
+    </review>
     <url>http://www.nbc.com</url>
   </programme>
 </tv>
@@ -657,7 +682,7 @@ if __name__ == '__main__':
                    'date': u'1995',
                    'desc': [(u'In an effort to grow up, George proposes marriage to former girlfriend Susan.',
                              u'')],
-                   'episode-num': (u'7 . 1 . 1/1', u'xmltv_ns'),
+                   'episode-num': [(u'7 . 1 . 1/1', u'xmltv_ns')],
                    'language': (u'English', u''),
                    'last-chance': (u'Hah!', u''),
                    'length': {'units': u'minutes', 'length': '22'},
@@ -671,10 +696,11 @@ if __name__ == '__main__':
                                          'width': u'64'}],
                                'system': u'VCHIP',
                                'value': u'PG'}],
-                   'star-rating': {'icon': [{'height': u'32',
+                   'review': [{'type': 'url', 'value': 'http://some.review/'}],
+                   'star-rating': [{'icon': [{'height': u'32',
                                              'src': u'http://some.star/icon.png',
                                              'width': u'32'}],
-                                   'value': u'4/5'},
+                                   'value': u'4/5'}],
                    'start': u'20030702000000 ADT',
                    'stop': u'20030702003000 ADT',
                    'sub-title': [(u'The Engagement', u'')],
